#include "stdafx.h"
#include "windows.h"
#include "avisynth.h"


class DePat : public GenericVideoFilter {
  // DePat defines the name of your filter class.
  // This name is only used internally, and does not affect the name of your filter or similar.
  // This filter extends GenericVideoFilter, which incorporates basic functionality.
  // All functions present in the filter must also be present here.


  PClip WindowVideo; // define the secondary video clip parameter variable
  int SquareSize; // define thesquare size parameter variable


public:
  // This defines that these functions are present in your class.
  // These functions must be that same as those actually implemented.
  // Since the functions are "public" they are accessible to other classes.
  // Otherwise they can only be called from functions within the class itself.

  DePat(PClip _child,      PClip _WindowVideo, int _SquareSize,  IScriptEnvironment* env);
  // This is the constructor. It does not return any value, and is always used,
  //  when an instance of the class is created.
  // Since there is no code in this, this is the definition.

  ~DePat();
  // The is the destructor definition. This is called when the filter is destroyed.


  PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
  // This is the function that AviSynth calls to get a given frame.
  // So when this functions gets called, the filter is supposed to return frame n.
};

class DePatDebug : public GenericVideoFilter {
  // DePatDebug defines the name of your filter class.
  // This name is only used internally, and does not affect the name of your filter or similar.
  // This filter extends GenericVideoFilter, which incorporates basic functionality.
  // All functions present in the filter must also be present here.


  PClip WindowVideo; // define the secondary video clip parameter variable

public:
  // This defines that these functions are present in your class.
  // These functions must be that same as those actually implemented.
  // Since the functions are "public" they are accessible to other classes.
  // Otherwise they can only be called from functions within the class itself.

  DePatDebug(PClip _child, PClip _WindowVideo, IScriptEnvironment* env);
  // This is the constructor. It does not return any value, and is always used,
  //  when an instance of the class is created.
  // Since there is no code in this, this is the definition.

  ~DePatDebug();
  // The is the destructor definition. This is called when the filter is destroyed.


  PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
  // This is the function that AviSynth calls to get a given frame.
  // So when this functions gets called, the filter is supposed to return frame n.
};


/***************************
 * The following is the implementation
 * of the defined functions.
 ***************************/

//Here is the acutal constructor code used
DePat::DePat(PClip _child, PClip _WindowVideo, int _SquareSize,  IScriptEnvironment* env) :
GenericVideoFilter(_child), WindowVideo(_WindowVideo), SquareSize(_SquareSize){
  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  //  where the following variables gets defined:
  //   PClip child;   // Contains the source clip.
  //   VideoInfo vi;  // Contains videoinfo on the source clip.
}

// This is where any actual destructor code used goes
DePat::~DePat() {
  // This is where you can deallocate any memory you might have used.
}




PVideoFrame __stdcall DePat::GetFrame(int n, IScriptEnvironment* env) {
  // This is the implementation of the GetFrame function.
  // See the header definition for further info.

  PVideoFrame src = child->GetFrame(n, env);
  // Request frame 'n' from the child (source) clip.
  PVideoFrame window = WindowVideo->GetFrame(n, env);
  // Request frame "'n" from the WindowVideo clip
  PVideoFrame dst = env->NewVideoFrame(vi);
  // Construct a frame based on the information of the current frame
  // contained in the "vi" struct.

  const unsigned char* srcp = src->GetReadPtr();
  // Request a Read pointer from the source frame.
  // This will return the position of the upperleft pixel in YUY2 images,
  // and return the lower-left pixel in RGB.
  // RGB images are stored upside-down in memory.
  // You should still process images from line 0 to height.

  unsigned char* dstp = dst->GetWritePtr();
  // Request a Write pointer from the newly created destination image.
  // You can request a writepointer to images that have just been
  // created by NewVideoFrame. If you recieve a frame from PClip->GetFrame(...)
  // you must call env->MakeWritable(&frame) be recieve a valid write pointer.

  const int dst_pitch = dst->GetPitch();
  // Requests pitch (length of a line) of the destination image.
  // For more information on pitch see: http://www.avisynth.org/index.php?page=WorkingWithImages
  // (short version - pitch is always equal to or greater than width to allow for seriously fast assembly code)

  const int dst_width = dst->GetRowSize();
  // Requests rowsize (number of used bytes in a line.
  // See the link above for more information.

  const int dst_height = dst->GetHeight();
  // Requests the height of the destination image.

  const int src_pitch = src->GetPitch();
  const int src_width = src->GetRowSize();
  const int src_height = src->GetHeight();

  const unsigned char* windowp=window->GetReadPtr();
  const int window_pitch = window->GetPitch();
  const int window_width = window->GetRowSize();
  const int window_height = window->GetHeight();
  // Get info on the Windowed Clip (see src definitions for more information)


  int w, h;

  if (!vi.IsRGB24()) {
    throw AvisynthError("MUST BE RGB24");
  }
  // The code just deals with RGB24 colourspace where each pixel is represented by
  // 3 bytes, Blue, Green and Red.
  // Although this colourspace is the easiest to understand, it is very rarely used because
  // a 3 byte sequence (24bits) cannot be processed easily using normal 32 bit registers.

  for (h=0; h < src_height;h++) {
  	for (w = 0; w < src_width; w+=3) {
	    *(dstp + w) = *(srcp + w);            // Blue
	    *(dstp + w + 1) = *(srcp + w + 1);    // Green
	    *( dstp + w + 2) = *(srcp + w + 2);   // Red
  	}
  	srcp = srcp + src_pitch;
	  dstp = dstp + dst_pitch;
  }
  // end copy src to dst

  //Now draw a white square in the middle of the frame
  // Normally you'd do this code within the loop above but here it is in a separate loop for clarity;

  dstp = dst->GetWritePtr();
  dstp = dstp + (dst_height/2 - SquareSize/2)*dst_pitch;
  for (h=0; h < SquareSize;h++) { // only scan 100 lines
    for (w = dst_width/2 - SquareSize*3/2; w < dst_width/2 + SquareSize*3/2; w+=3) {
      *(dstp + w) = 255;          // Set Blue to maximum value.
	    *(dstp + w + 1) = 255;      // and Green.
      *(dstp + w + 2) = 255;      // and Red - therefore the whole pixel is now white.
    }
    dstp = dstp + dst_pitch;
  }

  return dst;
}

//Here is the acutal constructor code used
DePatDebug::DePatDebug(PClip _child, PClip _WindowVideo,  IScriptEnvironment* env) :
GenericVideoFilter(_child), WindowVideo(_WindowVideo) {
  // This is the implementation of the constructor.
  // The child clip (source clip) is inherited by the GenericVideoFilter,
  //  where the following variables gets defined:
  //   PClip child;   // Contains the source clip.
  //   VideoInfo vi;  // Contains videoinfo on the source clip.
}

// This is where any actual destructor code used goes
DePatDebug::~DePatDebug() {
  // This is where you can deallocate any memory you might have used.
}




PVideoFrame __stdcall DePatDebug::GetFrame(int n, IScriptEnvironment* env) {
  // This is the implementation of the GetFrame function.
  // See the header definition for further info.

  PVideoFrame src = child->GetFrame(n, env);
  // Request frame 'n' from the child (source) clip.
  PVideoFrame window = WindowVideo->GetFrame(n, env);
  // Request frame "'n" from the WindowVideo clip
  PVideoFrame dst = env->NewVideoFrame(vi);
  // Construct a frame based on the information of the current frame
  // contained in the "vi" struct.

  const unsigned char* srcp = src->GetReadPtr();
  // Request a Read pointer from the source frame.
  // This will return the position of the upperleft pixel in YUY2 images,
  // and return the lower-left pixel in RGB.
  // RGB images are stored upside-down in memory.
  // You should still process images from line 0 to height.

  unsigned char* dstp = dst->GetWritePtr();
  // Request a Write pointer from the newly created destination image.
  // You can request a writepointer to images that have just been
  // created by NewVideoFrame. If you recieve a frame from PClip->GetFrame(...)
  // you must call env->MakeWritable(&frame) be recieve a valid write pointer.

  const int dst_pitch = dst->GetPitch();
  // Requests pitch (length of a line) of the destination image.
  // For more information on pitch see: http://www.avisynth.org/index.php?page=WorkingWithImages
  // (short version - pitch is always equal to or greater than width to allow for seriously fast assembly code)

  const int dst_width = dst->GetRowSize();
  // Requests rowsize (number of used bytes in a line.
  // See the link above for more information.

  const int dst_height = dst->GetHeight();
  // Requests the height of the destination image.

  const int src_pitch = src->GetPitch();
  const int src_width = src->GetRowSize();
  const int src_height = src->GetHeight();

  const unsigned char* windowp=window->GetReadPtr();
  const int window_pitch = window->GetPitch();
  const int window_width = window->GetRowSize();
  const int window_height = window->GetHeight();
  // Get info on the Windowed Clip (see src definitions for more information)


  int w, h;

  if (!vi.IsRGB24()) {
    throw AvisynthError("MUST BE RGB24");
  }
  // The code just deals with RGB24 colourspace where each pixel is represented by
  // 3 bytes, Blue, Green and Red.
  // Although this colourspace is the easiest to understand, it is very rarely used because
  // a 3 byte sequence (24bits) cannot be processed easily using normal 32 bit registers.

  for (h=0; h < src_height;h++) {
  	for (w = 0; w < src_width; w+=3) {
      unsigned char r, g, b;
      unsigned char r1, g1, b1;
      unsigned char r2, g2, b2;
      r1 = *(srcp + w);
      g1 = *(srcp + w + 1);
      b1 = *(srcp + w + 2);
      r2 = *(windowp + w);
      g2 = *(windowp + w + 1);
      b2 = *(windowp + w + 2);
	    *(dstp + w) = abs(r1 - r2);
	    *(dstp + w + 1) = abs(g1 - g2);
	    *( dstp + w + 2) = abs(b1 - b2);
  	}
  	srcp = srcp + src_pitch;
	  dstp = dstp + dst_pitch;
	  windowp = windowp + dst_pitch;
  }
  // end copy src to dst

  return dst;
}


// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,
// based on the arguments recieved.

AVSValue __cdecl Create_DePat(AVSValue args, void* user_data, IScriptEnvironment* env) {
  return new DePat(args[0].AsClip(), // the 0th parameter is the source clip
                          args[1].AsClip(), // Corresponds to our 1st parameter - the WindowedVideo
                          args[2].AsInt(0), // Corresponds to our 2nd parameter - the size of the square in pixels.
                          env);
  // Calls the constructor with the arguments provied.
}

AVSValue __cdecl Create_DePatDebug(AVSValue args, void* user_data, IScriptEnvironment* env) {
  return new DePatDebug(args[0].AsClip(),
                          args[1].AsClip(),
                          env);
  // Calls the constructor with the arguments provied.
}


// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
  env->AddFunction("DePat", "cc[SIZE]i", Create_DePat, 0);
  env->AddFunction("DePatDebug", "cc", Create_DePatDebug, 0);

  return "`DePat' DePat plugin";
}
